package exosite;  
import java.io.BufferedReader; 
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import com.ewon.ewonitf.EWException;
import com.ewon.ewonitf.ScheduledActionManager;
import com.ewon.ewonitf.SysControlBlock;
import com.hms_networks.americas.sc.datapoint.DataPoint;

public class ExositeService {
	
	private static final String token_file = "/usr/exosite_token.txt";
	//TODO: Split these out
	private static final String response_file = "/usr/reponse.txt";
	private static final String data_in_response_file = "/usr/data_in_response_file.txt";
	private static final String config_io_response_file = "/usr/config_io_response.txt";
	
	private String product_domain = "https://f3l8y59gbpqg00000.m2.exosite.io/";
	private String provision_endpoint = "provision/activate";
	private String data_in_endpoint = "onep:v1/stack/alias";
	private String historical_in_endpoint = "onep:v1/stack/record";

	
	public void saveToken(String token)
	{
		try 
		{
			BufferedWriter writer = new BufferedWriter(new FileWriter(token_file));
			writer.write(token);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void clear_config_io_response() throws IOException
	{
		File file = new File(config_io_response_file);
		if (file.exists())
			file.delete();
	}
	
	public String readToken()
	{
		String token = "";
		
		File checkFile = new File(token_file);
		if (!checkFile.exists()) return token;
		
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(token_file));
			token = reader.readLine();
		} catch (Exception e) {
			e.printStackTrace();
		}
		 
		return token;
	}
	
	public String provision()
	{
		String token = "";

		String URL = product_domain + provision_endpoint;
		int res = ResponseCode.NO_ERROR;
		String id = "";

        SysControlBlock Sys;
		try {
			Sys = new SysControlBlock(SysControlBlock.INF);
	        id = Sys.getItem("SerNum");
		} catch (EWException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		
		System.out.println("Provisioning on " + URL + " with " + id);
		
	    try {
		  res = ScheduledActionManager.RequestHttpX(URL, "POST", "", "id=" + id,"", response_file);
	    } catch (EWException e) {
	      res = ResponseCode.EWON_ERROR;
	    }
	    	    
	    if (res == ResponseCode.NO_ERROR)
		{
		    BufferedReader reader;
		    try {
				reader = new BufferedReader(new FileReader(response_file));
			    token = reader.readLine();
			    saveToken(token);
			} catch (Exception e) {
				e.printStackTrace();
			}
	    }
	    
	    return token;    
	}
	
	private void printFile(String filename)
	{
		File file = new File(filename);
		if (!file.exists()) return;
	    BufferedReader reader;
	    try {
			reader = new BufferedReader(new FileReader(filename));
		    System.out.println(filename + " contents: " + reader.readLine());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    public int syncConfigIO(String token)
    {
		String URL = product_domain + "onep:v1/stack/alias?config_io";
        String auth_header = "X-Exosite-CIK=" + token;
        String accept_header = "Accept=application/x-www-form-urlencoded";
        String if_modified_since_header = "If-Modified-Since=0"; // Doesn't return true if eWon has already acked, so 0 is always a 'safe' bet. Equivalent would be having a timestamp of the last time we checked, but it's not necessary
        String timeout_header = "Request-Timeout=2"; // Quick timeout and we'll just check on a cadence...could do long polling, don't want to block... 
        int res = ResponseCode.NO_ERROR;
        
        try {
			clear_config_io_response();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
        
        String headers =  if_modified_since_header + "&" + timeout_header + "&" + auth_header + "&" + accept_header;

	    try {
		  res = ScheduledActionManager.RequestHttpX(URL, "GET", headers, "","", config_io_response_file);
	    } catch (EWException e) {
	      res = ResponseCode.EWON_ERROR;
	    }

    	File file = new File(config_io_response_file);
    	
	    if (res == ResponseCode.NO_ERROR && file.exists())
		{
		    BufferedReader reader;
		    try {
				reader = new BufferedReader(new FileReader(config_io_response_file));
			    String config_io = reader.readLine();
			    headers=auth_header;
            	System.out.println("Updating config_io with: " + headers);
                res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, config_io,"", config_io_response_file);
			} catch (Exception e) {
				e.printStackTrace();
			}
	    }
	    else
	    {
	    	//System.out.println("Config_io_unchanged");
	    }
	    
	    return 0;
    }
	
	public int writeData(String data, String token)
	{
		String URL = product_domain + data_in_endpoint;
		int res = ResponseCode.NO_ERROR;
		
		String headers = "X-Exosite-CIK="+token;

	    try {
		  res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, "data_in=" + data,"", data_in_response_file);
	    } catch (EWException e) {
	      res = ResponseCode.EWON_ERROR;
	    }
	    
	    return res;
	}
	
	private boolean is_previous_complete_data_in(String currentTimestamp, String previousTimestamp, String data_in)
	{
		// data_in is 'empty' so it's not complete
		if (data_in == "{") return false;
		
		//If there's a new timestamp, the previous timestamp is completed,
		// Or, the index is the size, so we're on the last DataPoint
		if (currentTimestamp.compareTo(previousTimestamp) == 0) {
			return false;
		} else {
			return true;
		}
		
	}
	
	private boolean is_current_complete_data_in(int index, int size)
	{
		return index == size-1;
	}
	
	private String addDataPointToDataIn(String data_in, DataPoint dp)
	{
		if (data_in == "") 
			data_in = "{";
		//Add current datapoint
		if (data_in != "{") data_in = data_in + ",";
		data_in = data_in + "\"" + dp.getTagId() + "\"" + ":" + dp.getValueString();
		
		return data_in;
	}
	
	private String addDataInToPayload(String payload, String data_in, String data_in_timestamp)
	{
		String timestamp_us = Long.toString(Long.parseLong(data_in_timestamp)*1000000);
		data_in = data_in + "}"; //Close data_in 'object'
		payload = payload + "&" + timestamp_us + "=" + data_in;
		return payload;
	}
	
	public int writeHistoricalData(ArrayList data, String token)
	{
		String URL = product_domain + historical_in_endpoint; 
		int res = ResponseCode.NO_ERROR;
		
		String headers = "Accept: */*" + "&" + "X-Exosite-CIK="+token;
		
		String payload = "alias=data_in";
		
		String data_in_timestamp = ""; //Just a default
		String data_in = "";
		
		//<time1>={"001":1234}&
		System.out.println("Size: " + data.size());
		for(int i=0; i<data.size(); i++)
		{
			DataPoint dp = (DataPoint) data.get(i);
			if (data_in_timestamp == "") data_in_timestamp = getTimestampFromDP(dp);//set data_in ts
			if (is_previous_complete_data_in(getTimestampFromDP(dp), data_in_timestamp, data_in)) {
				payload = addDataInToPayload(payload, data_in, data_in_timestamp);
				data_in = ""; //Start fresh data_in 'object'
				data_in_timestamp = getTimestampFromDP(dp);
				data_in = addDataPointToDataIn(data_in, dp);
			} else if (is_current_complete_data_in(i, data.size())) {
				data_in = addDataPointToDataIn(data_in, dp);
				payload = addDataInToPayload(payload, data_in, data_in_timestamp);
				data_in = "";
				data_in_timestamp = ""; //reset so as to re-initialize with the next dp timestamp
			} else {
				data_in = addDataPointToDataIn(data_in, dp);
			}
		}
		//Payload format:
		//alias=<alias 1>&<timestamp 1>=<value 1>&<timestamp 2>=<value 2>&alias=<alias 2>&<timestamp 3>=<value 3>&<timestamp 4>=<value 4>
	    if (data.size() >0) // Don't send if there's no new data
	    {
			try {
			  //payload = "alias=data_in&1593635188979={\"001\":15}";
			  //payload = "alias=data_in&1593636617000000={\"001\":16}";

				                          
			  res = ScheduledActionManager.RequestHttpX(URL, "POST", headers, payload,"", data_in_response_file);
			} catch (EWException e) {
		      res = ResponseCode.EWON_ERROR;
		    }
	    }
	    else
	    {
	    	//System.out.println("No new data points");
	    }
	    
	    return res;
	}
	
	private String getTimestampFromDP(DataPoint dp)
	{
		String longTimestamp = dp.getTimeStamp();
		int timestamp_int = Integer.parseInt(longTimestamp)/1000; 
		return timestamp_int + "";
		
	}
}
